
#if __cplusplus
extern "C" 
#endif
double psub_Ice(double T);

#if __cplusplus
extern "C" 
#endif
double g_Ice(double T,double p);

#if __cplusplus
extern "C" 
#endif
double dg_dp_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double dg2_dp2_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double IsothermCompress_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double dg_dT_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double h_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double s_Ice(double T, double p);

#if __cplusplus
extern "C" 
#endif
double rho_Ice(double T, double p);